#!/usr/bin/env python2
# -*- coding: utf-8 -*-

class debugInp():
    def __init__(self):
        self.ebeam = 76.56296774
        self.pbeam = 4.88
        self.rtang = 1.715
        self.BT0 = 2.0
        self.nbeams_loc='/home/jonathan/Dropbox/GTEDGE/MyPrograms/GTEDGE/lib/beams/NBeamsMDS/NBeams/bin/Release/nbeams'
