#!/usr/bin/python
from scipy import constants

"""Physics-related constants"""
epsilon_0 = constants.epsilon_0
elementary_charge = constants.elementary_charge
electron_mass = constants.electron_mass
deuteron_mass = constants.physical_constants['deuteron mass'][0]
N_A = constants.N_A
mu_0 = constants.mu_0
MARKERSIZE=6